/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_LINETYPE_H
#define DPC_IMPL_TYPE_LINETYPE_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"
#include "impl_type_uint8.h"

namespace dpc {
struct LineType {
    ::Float start_offset{ 0.0F };
    ::Float end_offset{ 0.0F };
    ::UInt8 linetype{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(start_offset);
        fun(end_offset);
        fun(linetype);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(start_offset);
        fun(end_offset);
        fun(linetype);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("start_offset", start_offset);
        fun("end_offset", end_offset);
        fun("linetype", linetype);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("start_offset", start_offset);
        fun("end_offset", end_offset);
        fun("linetype", linetype);
    }

    friend bool operator==(const ::dpc::LineType& lhs, const ::dpc::LineType& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.start_offset - rhs.start_offset)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.end_offset - rhs.end_offset)) < DBL_EPSILON) && (lhs.linetype == rhs.linetype);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_LINETYPE_H
